Package: otel
Title: OpenTelemetry R API
Version: 0.2.0
Authors@R: 
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre"))
Description: High-quality, ubiquitous, and portable telemetry to enable
    effective observability. OpenTelemetry is a collection of tools,
    APIs, and SDKs used to instrument, generate, collect, and export
    telemetry data (metrics, logs, and traces) for analysis in order to
    understand your software's performance and behavior.
    This package implements the OpenTelemetry API:
    <https://opentelemetry.io/docs/specs/otel/>.
    Use this package as a dependency if you want to instrument your R
    package for OpenTelemetry.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2.9000
Depends: R (>= 3.6.0)
Suggests: callr, cli, glue, jsonlite, otelsdk, processx, shiny,
        spelling, testthat (>= 3.0.0), utils, withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
URL: https://otel.r-lib.org, https://github.com/r-lib/otel
Additional_repositories:
        https://github.com/r-lib/otelsdk/releases/download/devel
BugReports: https://github.com/r-lib/otel/issues
NeedsCompilation: no
Packaged: 2025-08-29 12:46:28 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2025-08-29 13:30:07 UTC
