# Generated by roxygen2: do not edit by hand

S3method(format,otel_counter)
S3method(format,otel_gauge)
S3method(format,otel_histogram)
S3method(format,otel_logger)
S3method(format,otel_logger_provider)
S3method(format,otel_meter)
S3method(format,otel_meter_provider)
S3method(format,otel_span)
S3method(format,otel_span_context)
S3method(format,otel_tracer)
S3method(format,otel_tracer_provider)
S3method(format,otel_up_down_counter)
S3method(print,otel_counter)
S3method(print,otel_gauge)
S3method(print,otel_histogram)
S3method(print,otel_logger)
S3method(print,otel_logger_provider)
S3method(print,otel_meter)
S3method(print,otel_meter_provider)
S3method(print,otel_span)
S3method(print,otel_span_context)
S3method(print,otel_tracer)
S3method(print,otel_tracer_provider)
S3method(print,otel_up_down_counter)
export(as_attributes)
export(counter_add)
export(default_tracer_name)
export(end_span)
export(extract_http_context)
export(gauge_record)
export(get_active_span)
export(get_active_span_context)
export(get_default_logger_provider)
export(get_default_meter_provider)
export(get_default_tracer_provider)
export(get_logger)
export(get_meter)
export(get_tracer)
export(histogram_record)
export(invalid_span_id)
export(invalid_trace_id)
export(is_logging_enabled)
export(is_measuring_enabled)
export(is_tracing_enabled)
export(local_active_span)
export(log)
export(log_debug)
export(log_error)
export(log_fatal)
export(log_info)
export(log_severity_levels)
export(log_trace)
export(log_warn)
export(logger_provider_noop)
export(meter_provider_noop)
export(pack_http_context)
export(span_kinds)
export(span_status_codes)
export(start_local_active_span)
export(start_span)
export(tracer_provider_noop)
export(up_down_counter_add)
export(with_active_span)
