## Getting started

```{r}
#| include: FALSE
gslink <- if (Sys.getenv("IN_PKGDOWN") == "true") {
  "reference/gettingstarted.html"
} else {
  "https://otel.r-lib.org/reference/gettingstarted.html"
}
```

Set `otel_tracer_name` to the desired tracer name. (See `?otel_tracer_name`.)
Then add `otel::start_local_active_span()` calls to the functions you want
to trace:

```r
otel_tracer_name <- "<package-id>"
fn <- function(...) {
  spn <- otel::start_local_active_span("fn")
  ...
}
```

See [Getting Started](`r gslink`) for details.
