# as_string

    Code
      as_string(s1)
    Condition
      Error in `as_string()`:
      ! Invalid argument: s1 must be a string scalar, but it is a number.
    Code
      as_string(s2)
    Condition
      Error in `as_string()`:
      ! Invalid argument: s2 must be a string scalar, but it is an empty character vector.
    Code
      as_string(s3)
    Condition
      Error in `as_string()`:
      ! Invalid argument: s3 must be a string scalar, but it is a character vector.
    Code
      as_string(s4, null = FALSE)
    Condition
      Error in `as_string()`:
      ! Invalid argument: s4 must be a string scalar, but it is NULL.

